/**
 * Holds utility codes.
 *
 * @module ctt
 * @submodule util
 */
(function() {
dixim.namespace('dixim.ctt.util');
}());

(function() {
/**
 * @class Utils
 * @static
 */
dixim.ctt.util.Utils = {
    getObjectSize: function(obj) {
        return Object.keys(obj).length;
    },
    getAppVar: function() {
        var option = window.location.search;
        var verFlag = 0;
    
        if (option !== '') {
            var optionList = option.substr(1).split('&');
            for (var i = 0; i < optionList.length; i++) {
                if (optionList[i] === 'app=1') verFlag = 1;
                if (optionList[i] === 'var=2') verFlag = 2;
            }
        }
        if (verFlag) return verFlag;
        return -1;
    },
    isUserAgent: function(user_agent) {
        var ua = {};
    
        ua.name = window.navigator.userAgent.toLowerCase();
        if (ua.name.indexOf(user_agent) >= 0) {
            return true;
        }
        return false;
    },
    isJqueryMobile: function() {
        if (this.getAppVar() == 2) {
            if (this.isUserAgent('ipad')) return false;
            if (this.isUserAgent('ipod')) return false;
            if (this.isUserAgent('iphone')) return false;
        }
        return true;
    },
    isBrowserMode: function() {
        var option = window.location.search;
    
        if (option !== '') {
            var optionList = option.substr(1).split('&');
            for (var i = 0; i < optionList.length ; i++){
                if (optionList[i] === 'app=1') return false;
            }
        }
        return true;
    },
    test: function(text, pattern) {
        return pattern.test(text);
    },
    getIpVersion: function() {
        var hostname = location.hostname;

        if (hostname.match(/^\[[0-9a-f]{0,4}:[0-9a-f]{0,4}:[0-9a-f]{0,4}:[0-9a-f]{0,4}:[0-9a-f]{0,4}:[0-9a-f]{0,4}\]$/g)) return 6;
        return 4;
    },
};
})();


(function() {
var _pad2 = function(num) {
    var s = num.toString();
    if (s.length === 1) return ['0', s].join('');
    return s;
};

var _getStringSize = function(str) {
    // Matches only the 10.. bytes that are non-initial characters in a multi-byte sequence.
    var ret = encodeURIComponent(str).match(/%[89ABab]/g);

    return str.length + (ret ? ret.length : 0);
};

/**
 * @class Formatter
 * @static
 */
dixim.ctt.util.Formatter = {
    /**
    * Stringifies Date object.
    *
    * YYYY/MM/DD hh:mm
    * YYYY/MM/DD hh:mm:ss
    *
    * @method stringifyDate
    * @static
    * @param {Date} d
    * @param {String} [f='YYYY/MM/DD hh:mm'] format. 'YYYY/MM/DD hh:mm' | 'YYYY/MM/DD hh:mm:ss'
    * @return {String}
    */
    stringifyDate: function(d, f) {
    var ymd;
    var time;

    ymd = [d.getFullYear(), _pad2(d.getMonth() + 1), _pad2(d.getDate())].join('/');
    time = [_pad2(d.getHours()), _pad2(d.getMinutes())];
    if (!!f && f === 'YYYY/MM/DD hh:mm:ss') {
        time.push(_pad2(d.getSeconds()));
    }

    return [ymd, time.join(':')].join(' ');
    },

    /**
    * Converts dc:date format string to Date object.
    *
    * @method dcDateToDate
    * @static
    * @param {String} dcDate
    * @return {Date}
    */
    dcDateToDate: function(dcDate) {
        var ret;
        var date;
        var re = /^([0-9]{4})-([0-9]{2})-([0-9]{2})T([0-9]{2}):([0-9]{2}):([0-9]{2})(\.([0-9]{3}))?$/;

        if (!!!(ret = re.exec(dcDate))) return null;
        date = new Date(
            parseInt(ret[1], 10),
            parseInt(ret[2], 10) - 1,
            parseInt(ret[3], 10),
            parseInt(ret[4], 10),
            parseInt(ret[5], 10),
            parseInt(ret[6], 10),
            parseInt(ret[7], 10) || 0
        );
        return date;
    },

    /**
    * Converts Date object to dc:date format string.
    *
    * @method dateToDcDate
    * @static
    * @param {Date} d
    * @return {String}
    */
    dateToDcDate: function(d) {
        return [
            [d.getFullYear(), _pad2(d.getMonth() + 1), _pad2(d.getDate())].join('-'),
            'T',
            [_pad2(d.getHours()), _pad2(d.getMinutes()), '00'].join(':'),
            '.000',
        ].join('');
    },

    /**
    * Converts Minute object to HH:MM format string.
    *
    * @method minuteToTime
    * @static
    * @param {Integer} m
    * @return {String}
    */
    minuteToTime: function(m) {
        return [Math.floor(m / 60), ':', m % 60].join('');
    },

    stringifySize: (function(){
        var i, KB, MB, GB, TB;
        i = 1024;
        KB = i;
        MB = KB * i;
        GB = MB * i;
        TB = GB * i;

        return function(value, digit) {
            var size;

            if (typeof value === 'undefined') return '';
            if (typeof digit === 'undefined') digit = 1;
            if (!!!(size = parseInt(value, 10))) return '';
            if (size >= TB) {
                size = (size / TB).toFixed(digit) + 'TB';
            } else if (size >= GB) {
                size = (size / GB).toFixed(digit) + 'GB';
            } else if (size >= MB) {
                size = (size / MB).toFixed(digit) + 'MB';
            } else if (size >= KB) {
                size = (size / KB).toFixed(digit) + 'KB';
            } else {
                size = size + 'B';
            }
            return size;
        };
    }()),

    /**
    * Truncate string to specified length with ellipsis.
    *
    * @method truncateString
    * @param {String} s
    * @param {Number} len
    * @param {String} [ellipsis]
    * @returns
    */
    truncateString: function(s, len, ellipsis) {
        var n;

        if (s.length <= len) return s;
        if (!!ellipsis) {
            n = len - _getStringSize(ellipsis);
        } else {
            n = len;
        }
        return [s.substring(0, n), ellipsis].join('');
    },

    /**
    * Get object length.
    *
    * @method getObjectSize
    * @param {Object} obj
    * @returns
    */
    getObjectSize: function(obj) {
        return Object.keys(obj).length;
    },
};

}());

(function() {
var _ = dixim.ctt.ResourceBundle.get;
var TEMPLATE_TIME_SELECTOR = '${hour}${min}';

/**
 * Holds HTML form generators.
 *
 * @class Form
 */

// TODO: when supporting the i18n, the order of the values
// should be localized accordingly.
dixim.ctt.util.Form = {
    /**
    * Creates time selector html.
    *
    * This method returns like the following two select HTML elements.
    *
    *     <!-- The '{suffix}' below will be replaced by
    *         'ctt_form_selector_time_hour' in ResourceBundle. -->
    *     <select name="hour">
    *       <option value="0" selected="selected">0{suffix}</option>
    *       <option value="1">1{suffix}</option>
    *       <option value="2">2{suffix}</option>
    *       <option value="3">3{suffix}</option>
    *       ...
    *       <option value="4">23</option>
    *     </select>
    *     <!-- The '{suffix}' below will be replaced by
    *         'ctt_form_selector_time_min' in ResourceBundle. -->
    *     <select name="min">
    *       <option value="0" selected="selected">00{suffix}</option>
    *       <option value="10">10{suffix}</option>
    *       <option value="20">20{suffix}</option>
    *       <option value="30">30{suffix}</option>
    *       <option value="40">40{suffix}</option>
    *       <option value="50">50{suffix}</option>
    *     </select>
    *
    * @method createTimeSelector
    * @param configs
    * @return {String}
    */
    createTimeSelector: function(configs) {
        var hourOpts = [];
        var minOpts = [];
        var i;
        var template;

        configs = configs || {};

        template = configs.template || TEMPLATE_TIME_SELECTOR;
        hourOpts.push('<label class="digit_box2"><select name="hour" data-role="none">');
        for (i = 0; i < 24; i += 1) {
            hourOpts.push([
                '<option value="', i, '">',
                ('00' + i).slice('-2'),
                '</option>',
            ].join(''));
        }
        hourOpts.push('</select></label>');
        hourOpts.push(_('form_selector_time_hour'));

        minOpts.push('<label class="digit_box2"><select name="min" data-role="none">');
        for (i = 0; i <= 50; i += 10) {
            minOpts.push([
                '<option value="', i, '">',
                (i || '00'),
                '</option>',
            ].join(''));
        }
        minOpts.push('</select></label>');
        minOpts.push(_('form_selector_time_min'));

        return template.
            replace('${hour}', hourOpts.join('')).
            replace('${min}', minOpts.join(''));
    },

    /**
    * Creates date selector HTML.
    *
    * This method returns like the following HTML.
    * Each option@value is UNIX time.
    *
    *     <select name="date">
    *       <option value="1337266644857">2012/5/17</option>
    *       <option value="1337353044857">2012/5/18</option>
    *       <option value="1337439444857">2012/5/19</option>
    *       <option value="1337525844857">2012/5/20</option>
    *       <option value="1337612244857">2012/5/21</option>
    *       <option value="1337698644857">2012/5/22</option>
    *       <option value="1337785044857">2012/5/23</option>
    *     </select>
    *
    * @param configs
    * @return {String}
    */
    createDateSelector: function(configs) {
        var today = new Date();
        var dates;
        var oneday = 86400000;
        var dateOpts = [];
        var date;
        var time;
        var i;
        var length;

        configs = configs || {};
        length = configs.length || 7;

        today.setHours(0);
        today.setMinutes(0);
        today.setSeconds(0);
        today.setMilliseconds(0);
        dates = [today];
        time = today.getTime();
        for (i = 1; i < length; i += 1) {
            dates.push(new Date(time + oneday * i));
        }

        dateOpts.push('<select name="date" data-role="none">');
        for (i = 0; i < length; i += 1) {
            date = dates[i];
            dateOpts.push([
                '<option value="', date.getTime(), '">',
                [
                    date.getFullYear(),
                    ('00' + (date.getMonth() + 1)).slice(-2),
                    ('00' + date.getDate()).slice(-2),
                ].join('/'),
                '</option>'
            ].join(''));
        }
        dateOpts.push('</select>');
        return dateOpts.join('');
    },
    /**
    * @method getNextDateRange
    * @param {Date} [date]
    * @return {Date}
    */
    getNextDateRange: function(date) {
        var min;
        var msec;
        var p;

        date = date || new Date();
        date = new Date(date.getTime() + 600000);
        date.setSeconds(0);
        date.setMilliseconds(0);
        min = date.getMinutes();
        msec = date.getTime();
        if (min && (p = min % 10)) {
            msec += 60000 * (10 - p);
        }
        return new Date(msec);
    }
};

}());

(function() {
var Configs = dixim.ctt.Configs;
var _ = dixim.ctt.ResourceBundle.get;

dixim.ctt.util.Template = {
    LOADING_ICON_SMALL: [
        '<span class="ctt-loading ctt-loading-small">',
        '<img src="',
        Configs.get('imagePath'),
        '/loading_s.gif',
        '" /></span>',
    ].join(''),

    LOADING_ICON_LARGE: [
        '<span class="ctt-loading ctt-loading-large">',
        '<img src="',
        Configs.get('imagePath'),
        '/loading_l.gif',
        '" /></span>',
    ].join(''),
    LOADING_OVERLAY: '<div class="ctt-loading-overlay"></div>',
    SEPARATE: (_('separate') === '') ? '\\' : _('separate'),
};

}());

(function() {
var $ = dixim.ctt.jQuery;

dixim.ctt.util.PlaceHolder = {
    set: function(selector, text) {
        $(selector).val(text);
        $(selector).css('color', '#999');
        $(selector).addClass('placeholding');
        $(selector).focus(function() {
            if ($(this).val() === text) {
                $(this).val('');
                $(this).css('color', 'black');
                $(this).removeClass('placeholding');
            }
        }).blur(function() {
            if ($(this).val() === '') {
                $(this).val(text);
                $(this).css('color', '#999');
                $(this).addClass('placeholding');
            }
        }).on('change', function() {
            if ($(this).val() !== '') {
                $(this).css('color', 'black');
                $(this).removeClass('placeholding');
            }
        });
    },
};
}());

(function() {
var Locker = function() {
    var locking = false;

    return {
        lock: function() {
            locking = true;
        },
        unlock: function() {
            locking = false;
        },
        isLock: function() {
            return locking;
        },
    };
};
dixim.ctt.util.Locker = Locker;
}());

(function() {
var PinManager = function() {
    var pin = null;

    return {
        get: function() {
            return pin;
        },
        set: function(p) {
            pin = p;
        },
    };
};

dixim.ctt.util.PinManager = new PinManager();
}());

(function() {
var UiFunctionManager = function() {
    var flags = {};

    return {
        set: function(key) {
            flags[key] = true;
        },
        isImplement: function(key) {
            return key in flags;
        },
    };
};

dixim.ctt.util.UiFunctionManager = new UiFunctionManager();
}());

(function() {
var Enum = function() {
    var enums = {
        SERVER_STATUS: {
            UNKNOWN: 0x0000,
            STARTED: 0x0001,
            STOPPED: 0x0002,
            STOPPING_SERVICE: 0x0004,
        },
        COND: {
            TITLE: 1,
            DESCRIPTION: 2,
            CHANNEL: 3,
            GENRE: 4,
            DURATION: 5,
            COPY_COUNT: 6,
            DATE: 7,
            DAY: 8,
            START_TIME: 9,
            END_TIME: 10,
            BROADCAST: 11,
        },
        DELETION: {
            TASK: 1,
            FILTER: 2,
            DMS: 3,
            CONTENTS: 4,
            LOG: 5,
            SHARED_FOLDER: 6,
        },
        PROGRESS: {
            PERMITTED: 1,
            NOT_PERMITTED: 2,
            AGGREGATION: 3,
            NON_AGGREGATION: 4,
            PAUSE: 5,
            RESTART: 6,
            SYNC_PUBLIC: 7,
        },
        MEDIA_TYPE: {
            PHOTO: 0x0001,
            MUSIC: 0x0002,
            VIDEO: 0x0004,
        },
        DIALOG: {
            EDIT: 0x0001,
            ADD: 0x0002,
        },
        EONKYO_STATUS: {
            WAITING: 1,
            DOWNLOADING: 2,
            DOWNLOADED: 4,
            ERROR: 8,
        },
        OPERATION_MENU: {
            NO_SELECTED: 0x0001,
            MULTIPLE: 0x0002,
            TASK: 0x0004,
            ITEM: 0x0008,
            CONTAINER: 0x0010,
            SEARCH_PAGE: 0x0020,
            CATEGORY_CONTENTS: 0x0040,
        },
    };

    return {
        fetch: function(name) {
            return enums[name];
        },
        isOnFlag: function(param1, param2) {
            return param1 & param2;
        },
        isOffFlag: function(param1, param2) {
            return (param1 | param2) !== param1;
        },
    };
};

dixim.ctt.util.Enum = new Enum();
}());

