(function() {
var BinaryBuffer;

BinaryBuffer = function(configs) {
    this._initialize(configs);
};
dixim.common.BinaryBuffer = BinaryBuffer;
prototype = BinaryBuffer.prototype;

prototype.length = 0;

prototype._initialize = function(size) {
    this.length = size;
    this.buffer_array = new ArrayBuffer(size);
    this.view = new DataView(this.buffer_array);
    this.r_offset = 0;
    this.w_offset = 0;
};

prototype.addByte = function(num) {
    var little_endian = true;

    this.view.setUint8(this.w_offset, num, little_endian);
    this.w_offset += 1;
};

prototype.addBytes = function(arr) {
    for (var i = 0; i < arr.length; i += 1) {
        this.addByte(arr[i]);
    }
};

prototype.seekReadOffset = function(offset) {
    this.r_offset = offset;
};

prototype.seekWriteOffset = function(offset) {
    this.w_offset = offset;
};

prototype.getByte = function() {
    var little_endian = false;
    var num;

    if (this.r_offset >= this.view.byteLength) return null;

    num = this.view.getUint8(this.r_offset, little_endian);
    this.r_offset += 1;

    return num;
};

prototype.getBytes = function(bytes) {
    var b;
    var arr = [];

    for (var i = 0; i < bytes; i += 1) {
        b = this.getByte();
        if (b === null) break;
        arr.push(b);
    }
    return arr;
};

prototype.getArrayBuffer = function() {
    return this.buffer_array;
};

})();

(function() {
var Decoder;
var BinaryBuffer = dixim.common.BinaryBuffer;

Decoder = function(configs) {
    this._initialize(configs);
};
dixim.common.Decoder = Decoder;
prototype = Decoder.prototype;

var BUFFER_SIZE = 40;

prototype._initialize = function(configs) {
    this.key = [
        0x93, 0x88, 0xC0, 0x14, 0xF2, 0x84, 0x88, 0x5A,
        0xAC, 0xD1, 0x4B, 0x55, 0x66, 0xC9, 0x68, 0xAF,
        0xDD, 0x25, 0x19, 0x0D, 0x0A, 0xB1, 0x61, 0xEF
    ];
    this.binary_buffer = new BinaryBuffer(BUFFER_SIZE);
    this.decoded_binary_buffer = new BinaryBuffer(BUFFER_SIZE);
};

prototype.base64FromArrayBuffer = function(ary_buffer) {
    var dic = [
        'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
        'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
        'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
        'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'
    ];
    var base64 = "";
    var ary_u8 = new Uint8Array( ary_buffer );
    var num = ary_u8.length;
    var n = 0;
    var b = 0;

    var i = 0;
    while(i < num){
        b = ary_u8[i];
        base64 += dic[(b >> 2)];
        n = (b & 0x03) << 4;
        i ++;
        if(i >= num) break;

        b = ary_u8[i];
        base64 += dic[n | (b >> 4)];
        n = (b & 0x0f) << 2;
        i ++;
        if(i >= num) break;

        b = ary_u8[i];
        base64 += dic[n | (b >> 6)];
        base64 += dic[(b & 0x3f)];
        i ++;
    }

    var m = num % 3;
    if(m) base64 += dic[n];
    if(m == 1){
        base64 += "==";
    }else if(m == 2){
        base64 += "=";
    }

    return base64;
};

prototype.base64ToArrayBuffer = function(base64) {
    var dic = new Object();
    dic[0x41]= 0; dic[0x42]= 1; dic[0x43]= 2; dic[0x44]= 3; dic[0x45]= 4; dic[0x46]= 5;
    dic[0x47]= 6; dic[0x48]= 7; dic[0x49]= 8; dic[0x4a]= 9; dic[0x4b]=10; dic[0x4c]=11;
    dic[0x4d]=12; dic[0x4e]=13; dic[0x4f]=14; dic[0x50]=15; dic[0x51]=16; dic[0x52]=17;
    dic[0x53]=18; dic[0x54]=19; dic[0x55]=20; dic[0x56]=21; dic[0x57]=22; dic[0x58]=23;
    dic[0x59]=24; dic[0x5a]=25; dic[0x61]=26; dic[0x62]=27; dic[0x63]=28; dic[0x64]=29;
    dic[0x65]=30; dic[0x66]=31; dic[0x67]=32; dic[0x68]=33; dic[0x69]=34; dic[0x6a]=35;
    dic[0x6b]=36; dic[0x6c]=37; dic[0x6d]=38; dic[0x6e]=39; dic[0x6f]=40; dic[0x70]=41;
    dic[0x71]=42; dic[0x72]=43; dic[0x73]=44; dic[0x74]=45; dic[0x75]=46; dic[0x76]=47;
    dic[0x77]=48; dic[0x78]=49; dic[0x79]=50; dic[0x7a]=51; dic[0x30]=52; dic[0x31]=53;
    dic[0x32]=54; dic[0x33]=55; dic[0x34]=56; dic[0x35]=57; dic[0x36]=58; dic[0x37]=59;
    dic[0x38]=60; dic[0x39]=61; dic[0x2b]=62; dic[0x2f]=63;
    var num = base64.length;
    var n = 0;
    var b = 0;
    var e;

    if(!num) return (new ArrayBuffer(0));
    if(num < 4) return null;
    if(num % 4) return null;

    e = num / 4 * 3;
    if(base64.charAt(num - 1) == '=') e -= 1;
    if(base64.charAt(num - 2) == '=') e -= 1;

    var ary_buffer = new ArrayBuffer(e);
    var ary_u8 = new Uint8Array(ary_buffer);
    var i = 0;
    var p = 0;
    while(p < e){
        b = dic[base64.charCodeAt(i)];
        if(b === undefined) return null;
        n = (b << 2);
        i ++;

        b = dic[base64.charCodeAt(i)];
        if(b === undefined) return null;
        ary_u8[p] = n | ((b >> 4) & 0x3);
        n = (b & 0x0f) << 4;
        i ++;
        p ++;
        if(p >= e) break;

        b = dic[base64.charCodeAt(i)];
        if(b === undefined) return null;
        ary_u8[p] = n | ((b >> 2) & 0xf);
        n = (b & 0x03) << 6;
        i ++;
        p ++;
        if(p >= e) break;

        b = dic[base64.charCodeAt(i)];
        if(b === undefined) return null;
        ary_u8[p] = n | b;
        i ++;
        p ++;
    }
    return ary_buffer;
};

prototype._sha256 = function(raw) {
    var sha_obj = new jsSHA(raw, "HEX");
    return sha_obj.getHash("SHA-256", "HEX");
};

prototype.getData = function(bytes) {
    return this.decoded_binary_buffer.getArrayBuffer();
};

prototype.setData = function(s_buffer_array) {
    var random;
    var s_view = new DataView(s_buffer_array);
    var little_endian = false;
    var test_buffer = new BinaryBuffer(BUFFER_SIZE);

    this.binary_buffer.seekWriteOffset(0);
    for (var i = 0; i < 8; i += 1) {
        random = Math.floor(Math.random() * 16);
        this.binary_buffer.addByte(random)
    }
    for (var i = 0; i < s_view.byteLength; i += 1) {
        this.binary_buffer.addByte(s_view.getUint8(i, little_endian));
        test_buffer.addByte(s_view.getUint8(i, little_endian));
    }
};

prototype.convertArrToHex = function(arr) {
    var str = "";

    for (var i = 0; i < arr.length; i += 1) {
        str += ("00" + arr[i].toString(16)).slice(-2);
    }
    return str;
};

prototype.convertHexToArr = function(hex) {
    var arr = [];

    for (var i = 0, j = 2; i < hex.length; i += 2, j += 2) {
        arr.push(parseInt(hex.slice(i, j), 16));
    }
    return arr;
};

prototype.xorArr = function(arr1, arr2) {
    var len = (arr1.length > arr2.length) ? arr2.length : arr1.length;
    var arr = [];

    for (var i = 0; i < len; i += 1) {
        arr.push(arr1[i] ^ arr2[i]);
    }
    return arr;
};

prototype.merge = function(arr1, arr2) {
    return $.merge($.merge([], arr1), arr2);
};

prototype.decode = function() {
    var e1;
    var ivn;

    this.binary_buffer.seekReadOffset(0);
    this.decoded_binary_buffer.seekWriteOffset(0);

    e1 = this.binary_buffer.getBytes(8);
    this.decoded_binary_buffer.addBytes(e1);
    ivn = this.convertArrToHex(this.merge(e1, this.key));

    while (ivn !== null) {
        ivn = this._decodeBlock(ivn);
    }
};

prototype._decodeBlock = function(iv) {
    var p = this.binary_buffer.getBytes(8);
    var ivn;
    var e;

    if (p.length === 0) return null;

    ivn = this._sha256(iv);
    e = this.xorArr(p, this.convertHexToArr(ivn).slice(0, 8));

    this.decoded_binary_buffer.addBytes(e);

    return ivn;
};

})();
