/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-wizard-timing-dialog-view
 */

(function() {
var TransferWizardTimingDialogView;
var prototype;
var superConstructor;
var ModalDialogView = dixim.common.ModalDialogView;
var Form = dixim.ctt.util.Form;
var Formatter = dixim.ctt.util.Formatter;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var $ = dixim.ctt.jQuery;

/**
 * @class TransferWizardTimingDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
TransferWizardTimingDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(TransferWizardTimingDialogView, ModalDialogView);
dixim.contents.TransferWizardTimingDialogView = TransferWizardTimingDialogView;
superConstructor = TransferWizardTimingDialogView.superConstructor;
prototype = TransferWizardTimingDialogView.prototype;

var NEXT = 'next';
var TEMPLATE_BODY_CONTENT = [
    '<div class="dlg_main_msg">',
    '<span>', '${description}', '</span>',
    '</div>',
    '<div id="reservation-description" class="reservation-description txt_gray">',
    '<span>', '${underDescription}', '</span>',
    '</div>',
    '<div id="select-reservation-howto">',
    '<label class="select-pointer">', '<input type="radio" name="reservation-howto-button" id="asap-download" value="ASAP" data-role="none" checked />',
    '<span class="timing-text">', '${timingAsap}', '</span>',
    '</label>',
    '<label class="select-pointer">','<input type="radio" name="reservation-howto-button" id="timed-download" value="TIMED" data-role="none" />',
    '<span class="timing-text">', '${timingTimed}', '</span>',
    '</div>',
    '<div id="select-reservation-date">',
    '<label class="date_box">', '${dateSelector}', '</label>',
    '<label class="digit_box2">', '${timeSelector}', '</label>',
    '</div>',
].join('');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'transfer-wizard-timing-dialog-'].join('');

    configs.bodyContent = TEMPLATE_BODY_CONTENT
        .replace('${description}', _('contents_transfer_wizard_timing_description'))
        .replace('${underDescription}', _('contents_transfer_wizard_timing_under_description'))
        .replace('${timingAsap}', _('contents_transfer_wizard_timing_asap'))
        .replace('${timingTimed}', _('contents_transfer_wizard_timing_timed'))
        .replace('${dateSelector}', Form.createDateSelector())
        .replace('${timeSelector}', Form.createTimeSelector({template: '${hour}</label><label class="digit_box2">${min}'}));

    configs.title = _('contents_operation_menu_transfer');
    configs.contentHeight = configs.contentHeight || '300px';
    configs.returnButton = dixim.proxy(this.hide, this);
    configs.doubleDialog = true;
    configs.buttons = [
        {text: _('dialog_next'), click: dixim.proxy(this._clickNext, this)},
    ];

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._render = function() {
    var getTime = $.now();
    var date = new Date(getTime);
    var hour = date.getHours();
    var minute = date.getMinutes();

    minute = Math.ceil(minute / 10) * 10;
    minute += 10;
    if (minute >= 60) {
        minute = minute - 60;
        hour = hour + 1;
        if (hour < 0) hour = 23;
    }
    $('#select-reservation-date [name="hour"]').val(hour);
    $('#select-reservation-date [name="min"]').val(minute);
    $('select[name="date"], select[name="hour"], select[name="min"]').prop('disabled', true);
    $('#select-reservation-date select').addClass('disable-select');
    $('#select-reservation-howto input[type="radio"]').on('click', this._timingToggle);

    superConstructor.prototype._render.call(this);
};

prototype._timingToggle = function() {
    if($(this).prop('id') === 'timed-download') {
        $('select[name="date"], select[name="hour"], select[name="min"]').prop('disabled', false);
        $('#select-reservation-date select').addClass('enable-select');
        $('#select-reservation-date select').removeClass('disable-select');
    } else {
        $('select[name="date"], select[name="hour"], select[name="min"]').prop('disabled', true);
        $('#select-reservation-date select').addClass('disable-select');
        $('#select-reservation-date select').removeClass('enable-select');
    }
};

prototype._clickNext = function() {
    var info = {};
    var date;
    var hour;
    var minute;
    var millisecond;

    if ($('#timed-download').is(':checked')) {
        date = $('#select-reservation-date [name="date"]').val();
        hour = $('#select-reservation-date [name="hour"]').val();
        minute = $('#select-reservation-date [name="min"]').val();
        millisecond = parseInt(date) + (parseInt(hour)*60*60*1000) + (parseInt(minute)*60*1000);
        info.startTime = Formatter.dateToDcDate(new Date(millisecond));
    }
    this.dispatchEvent(new Event(NEXT, info));
};

}());
