/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule parental-edit-pin-controller
 */

(function() {
var ParentalEditPinController;
var prototype;
var superConstructor;
var ParentalEditPinView = dixim.server.ParentalEditPinView;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var PinManager = dixim.ctt.util.PinManager;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var BlockUi = dixim.common.BlockUi;

/**
 * @class ParentalEditPinController
 * @constructor
 * @param {Object} configs
 */
ParentalEditPinController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ParentalEditPinController, EventTarget);
dixim.server.ParentalEditPinController = ParentalEditPinController;
prototype = ParentalEditPinController.prototype;
superConstructor = ParentalEditPinController.superConstructor;

var APPLY = 'apply';
var PREV = 'prev';
var UPDATE_PIN = 'updatePin';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'parental-edit-pin'].join('');

    this._model = new ApiClientTransfer();
    this._model.addEventListener('fetchPin', this._successFetchPin, this);
    this._model.addEventListener('fetchPinError', this._errorFetchPin, this);
    this._model.addEventListener('setPin', this._successSetPin, this);
    this._model.addEventListener('setPinError', this._errorSetPin, this);

    this._view = new ParentalEditPinView(configs);
    this._view.addEventListener(APPLY, this._clickApply, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._blockUi = new BlockUi();
    this._parentContext = configs.parentContext;
    this._candidateCode;
    this._ageLimit;
    this._pinCode = configs.pinCode;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._fetchPin();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._fetchPin = function() {
    var query = {};

    query.code = this._pinCode;
    this._model.fetchPin(query);
};

prototype._successFetchPin = function(e) {
    this._ageLimit = e.args.age_limit;
};

prototype._errorFetchPin = function() {
    this._ageLimit = null;
};

prototype._successSetPin = function() {
    if (PinManager.get() !== null) PinManager.set(this._candidateCode);
    this._pinCode = this._candidateCode;
    this._blockUi.unblock();
    this.dispatchEvent(new Event(UPDATE_PIN, {code: this._pinCode}));
};

prototype._errorSetPin = function() {
    this._blockUi.unblock();
    this._errorDialog(_('server_parental_edit_setting_could_not_pin'));
};

prototype._isMatch = function(x, y) {
    return (x.toString() === y.toString());
};

prototype._clickApply = function(e) {
    var query = {};
    var newCode = e.args.newCode;
    var confirmCode = e.args.confirmCode;

    this._blockUi.block();

    if (! this._isMatch(newCode, confirmCode)) {
        this._errorDialog(_('server_parental_edit_pin_error'));
        this._blockUi.unblock();
        return;
    }
    if (! newCode.match(/^\d{1,4}$/)) {
        this._errorDialog(_('server_parental_edit_pin_error'));
        this._blockUi.unblock();
        return;
    }
    this._candidateCode = newCode;

    while (typeof this._ageLimit === 'undefined') ;

    query.code = this._pinCode;
    query.code_new = newCode;
    query.age_limit = (this._ageLimit === 0) ? this._ageLimit : this._ageLimit - 3;

    this._model.setPin(query);
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('server_parental_menu_edit_pin_title');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
