/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule filter-list-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var ToolbarView = dixim.ctt.ToolbarView;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var ErrorDialog = dixim.ctt.ErrorDialog;

var FilterListDialogView;
var prototype;
var superConstructor;

/**
 * @class FilterListDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
FilterListDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(FilterListDialogView, ModalDialogView);
dixim.ctt.ad.FilterListDialogView = FilterListDialogView;
prototype = FilterListDialogView.prototype;
superConstructor = FilterListDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div class="ctt-setting-view-wrapper" />';
var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

var TEMPLATE_BODY_CONTENT = ['<div id="filter-ctt-toolbar-button"></div>',
                             '<div id="filter-list-table-header" class="table_header"></div>',
                             '<div id="filter"></div>'
                             ].join('');

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the edit button was clicked.
 *
 * @event edit
 */
var EDIT = 'edit';

/**
 * Triggered when the delete button was clicked.
 *
 * @event delete
 */
var DELETE = 'delete';

/**
 * Triggered when the selectAll button was clicked.
 *
 * @event selectAll
 */
var SELECTALL = 'selectAll';

/**
 * Triggered when the unselectAll button was clicked.
 *
 * @event unselectAll
 */
var UNSELECTALL = 'unselectAll';

/**
 * Triggered when the add button was clicked.
 *
 * @event add
 */
var ADD = 'add';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  configs  = configs || {};

  configs.title = configs.title || _('ad_dialog_filter_list_title');
  configs.contentBox = TEMPLATE_CONTENT_BOX;
  configs.bodyContent = TEMPLATE_BODY_CONTENT;
  configs.width = 'auto';
  configs.height = 'auto';

  this._model = new AutoDownloadApiClient();
  this._loadingIcon = $(TEMPLATE_LOADING);

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didCreateToolbarView = function() {
  this.toolbarView = new ToolbarView({
    containerNode: '#filter-ctt-toolbar-button',
    buttons: [
      {
        id: 'filter_edit',
        label: _("ad_dialog_filter_list_toolbar_edit")
      },
      {
        id: 'delete',
        label: _("ad_dialog_filter_list_toolbar_delete")
      },
      {
        id: 'select_all',
        label: _("ad_dialog_filter_list_toolbar_all_select")
      },
      {
        id: 'unselect_all',
        label: _("ad_dialog_filter_list_toolbar_all_unselect")
      },
      {
        id: 'filter_add',
        label: _("ad_dialog_filter_list_toolbar_add"),
        align: 'right'
      }
    ]
  });

  var _FilterListDidClickFilterEdit = function() {
    this.dispatchEvent(new Event(EDIT));
  };

  var _FilterListDidClickDelete = function() {
    this.dispatchEvent(new Event(DELETE));
  };

  var _FilterListDidClickSelectAll = function() {
    this.dispatchEvent(new Event(SELECTALL));
  };

  var _FilterListDidClickUnselectAll = function() {
    this.dispatchEvent(new Event(UNSELECTALL));
  };

  var _FilterListDidClickFilterAdd = function() {
    this.dispatchEvent(new Event(ADD));
  };

  var _setEventListeners = dixim.proxy(function() {
    this.toolbarView.addEventListener('filter_edit', _FilterListDidClickFilterEdit, this);
    this.toolbarView.addEventListener('delete', _FilterListDidClickDelete, this);
    this.toolbarView.addEventListener('select_all', _FilterListDidClickSelectAll, this);
    this.toolbarView.addEventListener('unselect_all', _FilterListDidClickUnselectAll, this);
    this.toolbarView.addEventListener('filter_add', _FilterListDidClickFilterAdd, this);
  }, this);
  this.toolbarView.show();
  _setEventListeners();
};

prototype._didCreateFilterList = function() {
  $('#filter-list-table-header').html(_("ad_dialog_filter_list_name"));

  $('#filter').html(
    [
      '<table id="filter-list" rules="rows">',
      '</table>'
    ].join('')
  );
};

prototype._didAddFilterTableRow = function(filter_data) {
  $('#filter-list').append(
    [
      '<tr><td class="filter_checkbox_td_class">',
      '<input type="checkbox" id="filter_checkbox_0" value="',filter_data.filterId,'">',
      '</td><td class="filter_name_row">',
      '<span id="filter_name_span">',filter_data.filterName,'</span>',
      '</td></tr>'
    ].join('')
  );
};

prototype._didCreateFilterTable = function() {
  var query = {};

  query.success = dixim.proxy(function(e) {
    var filters = e.args.filters;
    var filters_length = filters.length;

    for(var i = 0; i < filters_length; i += 1) {
      this._didAddFilterTableRow(filters[i]);
    }
  }, this);
  query.error = dixim.proxy(function() {
    this._showErrorDialog("ctt_error_dialog_default");
  }, this);
  query.async = false;
  this._model.fetchDownloadFilterList(query);
};

prototype._didRenderDialog = function(e) {
  this._didCreateToolbarView();
  this._didCreateFilterList();
  this._didCreateFilterTable();

  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
