/**
 * @module ctt
 * @submodule destination-storage-listing-wizard-page-controller
 */
(function() {
var Event = dixim.event.Event;
var WizardPageController = dixim.ctt.WizardPageController;
var DestinationStorageListingController = dixim.ctt.DestinationStorageListingController;
var DestinationStorageListingWizardPageView = dixim.ctt.DestinationStorageListingWizardPageView;
var ConfirmationWizardPageController = dixim.ctt.ConfirmationWizardPageController;
var ScheduledRecordingWizardPageController = dixim.ctt.ScheduledRecordingWizardPageController;
var _ = dixim.ctt.ResourceBundle.get;

var DestinationStorageListingWizardPageController;
var prototype;
var superConstructor;

/**
 * @class DestinationStorageListingWizardPageController
 * @extends WizardPageController
 * @uses DestinationStorageListingController
 * @uses DestinationStorageListingWizardPageView
 * @uses ContentsTransferApiClient
 * @uses Event
 * @constructor
 * @param {Object} configs
 */
DestinationStorageListingWizardPageController = function(configs) {
  WizardPageController.call(this, configs);
};
dixim.inherit(DestinationStorageListingWizardPageController, WizardPageController);
dixim.ctt.DestinationStorageListingWizardPageController = DestinationStorageListingWizardPageController;
prototype = DestinationStorageListingWizardPageController.prototype;
superConstructor = DestinationStorageListingWizardPageController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {ContentsTransferApiClient} configs.model
 */
prototype._initialize = function(configs) {
  superConstructor.prototype._initialize.call(this, configs);

  this._view = new DestinationStorageListingWizardPageView();
  this._destListingController = null;
};

prototype.start = function() {
  var v = this._view;

  v.addEventListener('render', this._didRenderView, this);
  v.addEventListener('cancel', this._didCancel, this);
  v.addEventListener('next', this._didClickNext, this);
  v.addEventListener('back', this._didClickBack, this);

  superConstructor.prototype.start.call(this);
  if (!!this._destListingController) {
    this._destListingController.start();
  }
};

prototype._didRenderView = function(e) {
  var dlc;
  var wizard;
  var dms;

  if (!!!(wizard = this._wizard)) {
    throw new Error('not pushed into wizard stack.');
  }
  model = wizard.getRegistryEntry('model');
  dms = wizard.getRegistryEntry('dms');

  dlc = new DestinationStorageListingController({
    containerNode: this._view.getBodyContent(),
    model: model,
    dms: dms
  });
  dlc.addEventListener('changeSelection', this._didChangeSelection, this);
  dlc.addEventListener('updateList', this._didUpdateList, this);
  dlc.addEventListener('updateListError', this._didFailUpdateList, this);

  this._destListingController = dlc;
};

prototype._didChangeSelection = function(e) {
  var dest;
  var v = this._view;

  dest = this._destListingController.getSelectedStorage();
  if (!!dest) {
    v.enableNextButton();
  } else {
    v.disableNextButton();
  }
};

prototype._didUpdateList = function() {
  var dlc = this._destListingController;
  var w = this._wizard;
  var dest;
  var num;

  num = dlc.getNumberOfStorages();
  if (!num) {
    this._skip();
    return;
  }

  if (!!!w) return;

  if (num === 1) {
    dlc.selectFirstStorage();
  } else if (!!(dest = w.getRegistryEntry('destStorage'))) {
    dlc.selectStorage(dest);
  }
};

prototype._didFailUpdateList = function() {
  this._view.showError(_('ctt_destination_storage_listing_wizard_page_view_error'));
};

prototype._didClickNext = function(e) {
  var w = this._wizard;
  var dest;
  var vc;

  if (!!(dest = this._destListingController.getSelectedStorage())) {
    w.setRegistryEntry('destStorage', dest);
  }
  vc = this._getNextViewController();
  w.pushViewController(vc);
};

prototype._getNextViewController = function() {
  var w = this._wizard;
  var dms;
  var schedule;

  schedule = w.getRegistryEntry('enableScheduledTransferring');
  dms = w.getRegistryEntry('dms');
  if (!!dms.moveRes && schedule) {
    return new ScheduledRecordingWizardPageController();
  } else {
    return new ConfirmationWizardPageController();
  }
};

prototype._skip = function() {
  var vc;
  var past;

  vc = this._getNextViewController();
  past = this._wizard.replaceTopViewController(vc);
  past.destroy();
};

prototype._didClickBack = function(e) {
  var vc;
  var wizard = this._wizard;

  wizard.setRegistryEntry('destStorage', null);
  vc = wizard.popViewController();
  vc.destroy();
};

prototype.stop = function() {
  if (!!this._destListingController) {
    this._destListingController.stop();
  }
  superConstructor.prototype.stop.call(this);
};

prototype.destroy = function() {
  if (!!this._destListingController) {
    this._destListingController.destroy();
  }
  superConstructor.prototype.destroy.call(this);
};

}());
